import os
import subprocess
import argparse
import sys

class BackupLVMUtils(object):
    def __init__(self):
        self.prefix_backup_datadir = '/clusterdata/backup/'
        self.temp_mount_datadir = '/mnt'

    def exists_lvm(self, lvm_fullname=""):
        if os.path.exists(lvm_fullname.rstrip('/')):
            return True
        else:
            return False

    def create_backupdir(self, stackname=""):
        backup_dir = self.prefix_backup_datadir.rstrip('/') + '/' + stackname
        try:
            if not os.path.exists(backup_dir):
                os.makedirs(backup_dir)
            return True
        except Exception as e:
            print(e)
            return False

    # 创建临时的数据挂载目录 （用来挂载，备份打包）
    def create_temp_mount_datadir(self, stackname=""):
        temp_mount_datadir = self.temp_mount_datadir.rstrip('/') + '/' + stackname
        try:
            if not os.path.exists(temp_mount_datadir):
                os.makedirs(temp_mount_datadir)
            return True
        except Exception as e:
            print(e)
            return False

    # 挂载 LVM 逻辑卷到临时的数据目录
    def mount_lvm_to_temp_datadir(self, stackname="", lvm_fullname=""):
        lvm_snapshot_fullname = lvm_fullname.rstrip('/')
        temp_mount_datadir = self.temp_mount_datadir.rstrip('/') + '/' + stackname
        cmd = "mount -o discard /dev/{block_device} {mount_path}".format(block_device=lvm_snapshot_fullname,
                                                                    mount_path=temp_mount_datadir)
        res = subprocess.getstatusoutput(cmd)
        res_code = res[0]
        # res_content = res[1]
        if int(res_code) == 0:
            return True
        else:
            return False

    def pack_filedir_to_backupdir(self, stackname="", timeout=3600, backup_filename=None):
        temp_mount_datadir = self.temp_mount_datadir.rstrip('/') + '/' + stackname
        backup_datadir = self.prefix_backup_datadir.rstrip('/') + '/' + stackname

        if backup_filename:
            backup_filename = backup_filename
        else:
            backup_filename = stackname + '.whole-manual.tar.gz'

        backup_file_fullpath = backup_datadir.rstrip('/') + '/' + backup_filename

        cmd = 'tar -pczvf {backup_file_fullpath}  -C {data_dir} . '.format(backup_file_fullpath=backup_file_fullpath,
                                                                           data_dir=temp_mount_datadir)
        try:
            res = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                 universal_newlines=True, timeout=timeout)
            if res.returncode == 0:
                return True

            else:
                return False
        except Exception as e:
            print("export backup file timeout")
            return False


    def get_lvm_info(self, stackname=""):
        cmd = 'lvs  -o lv_size,lv_full_name 2>/dev/null |grep -w {stackname}|head -n 1'.format(stackname=stackname)
        res = subprocess.getstatusoutput(cmd)
        res_code = res[0]
        if res[1] == '':
            print("LVM [{lvm_name}] 不存在".format(lvm_name=stackname))
        else:
            res_content = res[1].strip().split()
            if int(res_code) == 0:
                lvm_name = res_content[1]
                lvm_size = res_content[0]
                kwargs = {
                    "lvm_size": lvm_size,
                    "lvm_name": lvm_name,
                }
                return kwargs


    def backuplvm(self, stackname=""):
        # 1. 检查 LVM 是否存在
        lvm_info = self.get_lvm_info(stackname)
        print("lvm_info:", lvm_info)
        if not lvm_info:
            return False

        # 2. 创建备份目录
        if not self.create_backupdir(stackname):
            print("创建备份目录失败, 目录: {backup_dir}".format(backup_dir=self.prefix_backup_datadir.rstrip('/') + '/' + stackname))
            # return False

        # 3. 创建临时挂载目录
        if not self.create_temp_mount_datadir(stackname):
            print("创建临时挂载目录失败, 目录: {temp_mount_datadir}".format(temp_mount_datadir=self.temp_mount_datadir.rstrip('/') + '/' + stackname))
            # return False

        # 4. 挂载 LVM 到临时挂载目录
        if not self.mount_lvm_to_temp_datadir(stackname, lvm_info['lvm_name']):
            print("挂载 LVM 到临时挂载目录失败, LVM: {lvm_name}, 临时挂载目录: {temp_mount_datadir}".format(
                lvm_name=lvm_info['lvm_name'],
                temp_mount_datadir=self.temp_mount_datadir.rstrip('/') + '/' + stackname))
            # return False

        # 5. 打包临时挂载目录到备份目录
        if not self.pack_filedir_to_backupdir(stackname):
            print("打包临时挂载目录到备份目录失败, 临时挂载目录: {temp_mount_datadir}, 备份目录: {backup_dir}".format(
                temp_mount_datadir=self.temp_mount_datadir.rstrip('/') + '/' + stackname,
                backup_dir=self.prefix_backup_datadir.rstrip('/') + '/' + stackname))
            return False

        return True



def main():
    parser = argparse.ArgumentParser(
        description="对指定 LVM 逻辑卷进行备份"
    )
    parser.add_argument(
        "stackname",
        help="要备份的 LVM 逻辑卷名称，例如：mssql-9984"
    )
    args = parser.parse_args()

    backup_lvm = BackupLVMUtils()
    # 将 stackname 作为参数传给 backuplvm()
    res = backup_lvm.backuplvm(args.stackname)
    print(res)
    if res:
        print(f"{args.stackname} 备份成功")
    else:
        print(f"{args.stackname} 备份失败")
        sys.exit(1)


if __name__ == '__main__':
    main()
